'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.system = exports.need = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var need = exports.need = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
    var _ref2 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    var nodeRange = _ref2.nodeRange;
    var platform = _ref2.platform;
    var arch = _ref2.arch;
    var forceDownload = _ref2.forceDownload;
    var forceBuild = _ref2.forceBuild;
    var nodeVersions, nodeVersion, downloaded, built, downloadFailed, remote;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (nodeRange) {
              _context.next = 2;
              break;
            }

            throw (0, _log.wasReported)('nodeRange not specified');

          case 2:
            if (platform) {
              _context.next = 4;
              break;
            }

            throw (0, _log.wasReported)('platform not specified');

          case 4:
            if (arch) {
              _context.next = 6;
              break;
            }

            throw (0, _log.wasReported)('arch not specified');

          case 6:

            nodeRange = (0, _system.abiToNodeRange)(nodeRange); // 'm48' -> 'node6'

            if (!(nodeRange !== 'latest')) {
              _context.next = 11;
              break;
            }

            if (/^node/.test(nodeRange)) {
              _context.next = 10;
              break;
            }

            throw (0, _log.wasReported)('nodeRange must start with \'node\'');

          case 10:
            nodeRange = 'v' + nodeRange.slice(4); // 'node6' -> 'v6' for semver

          case 11:

            platform = (0, _system.toFancyPlatform)(platform); // win32 -> win
            arch = (0, _system.toFancyArch)(arch); // ia32 -> x86

            nodeVersions = (0, _keys2.default)(_patches2.default).filter(function (nodeVersion) {
              return _semver2.default.satisfies(nodeVersion, nodeRange) || nodeRange === 'latest';
            }).sort(function (nv1, nv2) {
              return _semver2.default.gt(nv1, nv2);
            });

            if (nodeVersions.length) {
              _context.next = 16;
              break;
            }

            throw (0, _log.wasReported)('No available node version satisfies \'' + nodeRange + '\'');

          case 16:
            nodeVersion = nodeVersions.pop();
            downloaded = (0, _places.localPlace)({ from: 'downloaded', arch: arch, nodeVersion: nodeVersion, platform: platform, version: _package.version });
            built = (0, _places.localPlace)({ from: 'built', arch: arch, nodeVersion: nodeVersion, platform: platform, version: _package.version });
            downloadFailed = void 0;

            if (forceBuild) {
              _context.next = 25;
              break;
            }

            _context.next = 23;
            return (0, _fsPromise.exists)(downloaded);

          case 23:
            if (!_context.sent) {
              _context.next = 25;
              break;
            }

            return _context.abrupt('return', downloaded);

          case 25:
            if (forceDownload) {
              _context.next = 31;
              break;
            }

            _context.next = 28;
            return (0, _fsPromise.exists)(built);

          case 28:
            if (!_context.sent) {
              _context.next = 31;
              break;
            }

            if (forceBuild) _log.log.info('Reusing base binaries built locally:', built);
            return _context.abrupt('return', built);

          case 31:
            if (forceBuild) {
              _context.next = 39;
              break;
            }

            _log.log.info('Downloading base binaries to:', _path2.default.dirname(downloaded));
            remote = (0, _places.remotePlace)({ arch: arch, nodeVersion: nodeVersion, platform: platform, version: _package.version });
            _context.next = 36;
            return (0, _cloud.download)(remote, downloaded);

          case 36:
            if (!_context.sent) {
              _context.next = 38;
              break;
            }

            return _context.abrupt('return', downloaded);

          case 38:
            downloadFailed = true;

          case 39:
            if (downloadFailed) _log.log.info('Not found in GitHub releases. Building from source...');
            if (forceBuild) _log.log.info('Building base binary from source:', _path2.default.basename(built));
            _context.next = 43;
            return (0, _build2.default)(nodeVersion, arch, built);

          case 43:
            return _context.abrupt('return', built);

          case 44:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function need(_x) {
    return _ref.apply(this, arguments);
  };
}();

var _system = require('./system.js');

var system = _interopRequireWildcard(_system);

var _places = require('./places.js');

var _log = require('./log.js');

var _build = require('./build.js');

var _build2 = _interopRequireDefault(_build);

var _cloud = require('./cloud.js');

var _fsPromise = require('fs-promise');

var _patches = require('../patches/patches.json');

var _patches2 = _interopRequireDefault(_patches);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _package = require('../package.json');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.system = system;